require "import"
import "android.content.Context"
import "android.content.Intent"
import "android.os.Handler"
import "android.speech.SpeechRecognizer"
import "android.speech.RecognitionListener"
import "android.speech.RecognizerIntent"
import "java.util.Locale"
import "android.net.Uri"
import "android.widget.LinearLayout"
import "android.widget.TextView"

local substitutionTable = {
    ["வணக்கம்"] = "Hello",
}

function capitalizeFirstWord(text)
    local firstChar = text:sub(1, 1)
    local restOfString = text:sub(2)
    return firstChar:upper() .. restOfString
end

function substituteWords(text)
    for word, substitution in pairs(substitutionTable) do
        text = text:gsub("%f[%a]" .. word .. "%f[%A]", substitution)
    end
    return text
end

function addPunctuation(text)
    local lastChar = text:sub(-1)
    local punctuation = { ".", "!", "?" }
    local hasPunctuation = false

    for _, p in ipairs(punctuation) do
        if lastChar == p then
            hasPunctuation = true
            break
        end
    end

    if not hasPunctuation then
        text = text .. ". "
    else
        text = text .. " "
    end

    return text
end

function hasInternetConnection()
    local connectivityManager = this.getSystemService(Context.CONNECTIVITY_SERVICE)
    local networkInfo = connectivityManager.getActiveNetworkInfo()
    return networkInfo ~= nil and networkInfo.isConnected()
end

function kill()
    if speechRecognizer then
        speechRecognizer.destroy()
        speechRecognizer = nil
    end
end

function startListening()
    if not hasInternetConnection() then
        service.asyncSpeak("Please check your internet connection and try again.")
        return false
    end

    local verbalizationDetected = false

    speechRecognizer = SpeechRecognizer.createSpeechRecognizer(this)
    speechListener = RecognitionListener{
        onResults = function(results)
            local data = results.getParcelableArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
            if data and data.size() > 0 then
                verbalizationDetected = true
                local recognizedText = data.get(0)
                recognizedText = capitalizeFirstWord(recognizedText)
                recognizedText = substituteWords(recognizedText)
                recognizedText = addPunctuation(recognizedText)

                local node = service.getEditText()
                service.insertText(node, recognizedText)
                service.speak(recognizedText)

                service.handler.postDelayed(function()
                    startListening()
                end, 3000)
            end

            if not verbalizationDetected then
                kill()
                return false
            end
        end,
        onError = function()
            kill()
            return false
        end
    }

    local recognizerIntent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, Locale.getDefault().toString())
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_CALLING_PACKAGE, this.getPackageName())
    speechRecognizer.startListening(recognizerIntent)
    speechRecognizer.setRecognitionListener(speechListener)

    return true
end

-- Check if the dialog has been shown before
local prefs = this.getSharedPreferences("MyPrefs", Context.MODE_PRIVATE)
local firstRun = prefs.getBoolean("firstRun", true)

if firstRun then
    -- Define and use the layout properly
    local layout = LinearLayout(this)
    layout.setOrientation(LinearLayout.VERTICAL)

    -- Show the dialog
    local dlg = LuaDialog(service)
    dlg.setButton("connect on telegram", function()
        -- Open the link when cancel button is clicked
        local intent = Intent(Intent.ACTION_VIEW, Uri.parse("https://t.me/jieshuofiles"))
        this.startActivity(intent)
    end)

    this.postSpeak(2000, "modded by r . d . s .! This dialog will only show you the first time.")
    dlg.setTitle("voice dictation English")
    dlg.View = layout
    dlg.show()

    -- Update the shared preference to indicate that the dialog has been shown
    local editor = prefs.edit()
    editor.putBoolean("firstRun", false)
    editor.apply()
else
    -- Start speech recognition
    startListening()
end